@@ -216,7 +216,7 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "merge bar http://bar.org")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=1)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual(None, location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org")
 
@@ -225,7 +225,7 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "nest bar http://bar.org baz")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=1)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org")
 
@@ -234,10 +234,10 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "nest bar http://bar.org baz\nmerge zam lp:zam")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=2)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org")
-        child_branch, location = base_branch.child_branches[1]
+        child_branch, location = base_branch.child_branches[1].as_tuple()
         self.assertEqual(None, location)
         self.check_recipe_branch(child_branch, "zam", "lp:zam")
 
@@ -246,10 +246,10 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "merge zam lp:zam\nnest bar http://bar.org baz")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=2)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual(None, location)
         self.check_recipe_branch(child_branch, "zam", "lp:zam")
-        child_branch, location = base_branch.child_branches[1]
+        child_branch, location = base_branch.child_branches[1].as_tuple()
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org")
 
@@ -258,11 +258,11 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "nest bar http://bar.org baz\n  merge zam lp:zam")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=1)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org",
                 num_child_branches=1)
-        child_branch, location = child_branch.child_branches[0]
+        child_branch, location = child_branch.child_branches[0].as_tuple()
         self.assertEqual(None, location)
         self.check_recipe_branch(child_branch, "zam", "lp:zam")
 
@@ -271,11 +271,11 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "nest bar http://bar.org baz\n  nest zam lp:zam zoo")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=1)
-        child_branch, location = base_branch.child_branches[0]
+        child_branch, location = base_branch.child_branches[0].as_tuple()
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org",
                 num_child_branches=1)
-        child_branch, location = child_branch.child_branches[0]
+        child_branch, location = child_branch.child_branches[0].as_tuple()
         self.assertEqual("zoo", location)
         self.check_recipe_branch(child_branch, "zam", "lp:zam")
 
@@ -286,11 +286,13 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "merge zam lp:zam 2")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=2, revspec="revid:a")
-        child_branch, location = base_branch.child_branches[0]
+        instruction = base_branch.child_branches[0]
+        child_branch = instruction.recipe_branch
+        location = instruction.nest_path
         self.assertEqual("baz", location)
         self.check_recipe_branch(child_branch, "bar", "http://bar.org",
                 revspec="tag:b")
-        child_branch, location = base_branch.child_branches[1]
+        child_branch, location = base_branch.child_branches[1].as_tuple()
         self.assertEqual(None, location)
         self.check_recipe_branch(child_branch, "zam", "lp:zam", revspec="2")
 
@@ -300,7 +302,7 @@ class RecipeParserTests(TestCaseInTempDir):
                 + "run touch test \n")
         self.check_base_recipe_branch(base_branch, "http://foo.org/",
                 num_child_branches=1)
-        child_branch, command = base_branch.child_branches[0]
+        child_branch, command = base_branch.child_branches[0].as_tuple()
         self.assertEqual(None, child_branch)
         self.assertEqual("touch test", command)
 
@@ -350,7 +352,7 @@ class BuildTreeTests(TestCaseWithTransport):
     def test_build_tree_single_branch_existing_branch(self):
         source = self.make_branch_and_tree("source")
         revid = source.commit("one")
-        target = self.make_branch_and_tree("target")
+        self.make_branch_and_tree("target")
         base_branch = BaseRecipeBranch("source", "1", 0.2)
         build_tree(base_branch, "target")
         self.failUnlessExists("target")
@@ -446,7 +448,7 @@ class BuildTreeTests(TestCaseWithTransport):
         base_branch = BaseRecipeBranch("source1", "1", 0.2)
         merged_branch = RecipeBranch("merged", "source2")
         base_branch.merge_branch(merged_branch)
-        e = self.assertRaises(errors.BzrCommandError, build_tree,
+        self.assertRaises(errors.BzrCommandError, build_tree,
                 base_branch, "target")
         self.failUnlessExists("target")
         tree = workingtree.WorkingTree.open("target")
@@ -702,7 +704,7 @@ class ResolveRevisionsTests(TestCaseWithTransport):
 
     def test_changed_command(self):
         source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
+        source.commit("one")
         branch1 = BaseRecipeBranch("source", "{revno}", 0.2)
         branch2 = BaseRecipeBranch("source", "{revno}", 0.2)
         branch1.run_command("touch test1")
@@ -714,7 +716,7 @@ class ResolveRevisionsTests(TestCaseWithTransport):
     def test_substitute(self):
         source =self.make_branch_and_tree("source")
         revid1 = source.commit("one")
-        revid2 = source.commit("two")
+        source.commit("two")
         branch1 = BaseRecipeBranch("source",
                 "{revno}-{revno:packaging}", 0.2, revspec="1")
         branch2 = RecipeBranch("packaging", "source")
@@ -725,6 +727,16 @@ class ResolveRevisionsTests(TestCaseWithTransport):
         self.assertEqual("1", branch1.revspec)
         self.assertEqual("1-2", branch1.deb_version)
 
+    def test_substitute_supports_debupstream(self):
+        # resolve_revisions should leave debupstream parameters alone and not
+        # complain.
+        source =self.make_branch_and_tree("source")
+        source.commit("one")
+        source.commit("two")
+        branch1 = BaseRecipeBranch("source", "{debupstream}-{revno}", 0.2)
+        resolve_revisions(branch1)
+        self.assertEqual("{debupstream}-2", branch1.deb_version)
+
 
 class BuildManifestTests(TestCaseInTempDir):
 
